/*
 * PmodCLS.c
 *
 *  Created on: Nov 13, 2017
 *      Author: Krzysztof Chojnowski
 */

#include "stm32l4xx_hal.h"

#include "PmodCLS.h"

SPI_HandleTypeDef pmodClsSpi;

/**
 * @brief Helper function for writing data over the SPI.
 */
static void write(char* data, uint32_t len)
{
  HAL_SPI_Transmit(&pmodClsSpi, (uint8_t*)data, len, 200);
}

void PmodCLS_Config(void)
{
  // Enable clock for all GPIO ports required for Pmod connector configuration.
  __HAL_RCC_SPI1_CLK_ENABLE();

  // Configure the SPI peripheral for 2 data lines and Chip Select pin managed by hardware.
  pmodClsSpi.Instance = SPI1;
  pmodClsSpi.Init.Mode = SPI_MODE_MASTER;
  pmodClsSpi.Init.Direction = SPI_DIRECTION_2LINES;
  pmodClsSpi.Init.DataSize = SPI_DATASIZE_8BIT;
  pmodClsSpi.Init.CLKPolarity = SPI_POLARITY_LOW;
  pmodClsSpi.Init.CLKPhase = SPI_PHASE_1EDGE;
  pmodClsSpi.Init.NSS = SPI_NSS_HARD_OUTPUT;
  pmodClsSpi.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_128;
  pmodClsSpi.Init.FirstBit = SPI_FIRSTBIT_MSB;
  pmodClsSpi.Init.TIMode = SPI_TIMODE_DISABLE;
  pmodClsSpi.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  pmodClsSpi.Init.NSSPMode = SPI_NSS_PULSE_ENABLE;

  HAL_SPI_Init(&pmodClsSpi);

  // PmodCLS configuration data contains the escape byte (0x1B) followed by the '[' character,
  // numerical arguments (if required) and command byte at the end.
  write("\x1b[j", 3);     // Clear the display and home cursor.
  write("\x1b[0h", 4);    // Set the display mode (16 characters wrap)
  write("\x1b[0c", 4);    // Set cursor mode (cursor off).
  write("\x1b[0;0H", 6);  // Set the cursor position to 0,0.
}

void HAL_SPI_MspInit(SPI_HandleTypeDef *hspi)
{
  // Initialize GPIO used by the SPI1 peripheral.
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();
  __HAL_RCC_GPIOE_CLK_ENABLE();

  // Configure all SPI pins in alternate mode.
  GPIO_InitTypeDef GPIO_InitStruct;

  GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF5_SPI1;
  GPIO_InitStruct.Pin = GPIO_PIN_1 | GPIO_PIN_7;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  GPIO_InitStruct.Pin = GPIO_PIN_14;
  HAL_GPIO_Init(GPIOE, &GPIO_InitStruct);

  GPIO_InitStruct.Pull = GPIO_PULLUP;
  GPIO_InitStruct.Pin = GPIO_PIN_0;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
}

void PmodCLS_Write(uint8_t line, char* text, uint32_t len)
{
  // Verify the arguments according to the defined limits.
  if(line >= MAX_LINES || len > MAX_LINE_CHARACTERS)
    return;

  // Set the cursor position to the beginning of the selected line..
  if(line == 0)
    write("\x1b[0;0H", 6);
  else
    write("\x1b[1;0H", 6);

  write(text, len);
}
